﻿using System;
using System.Windows.Input;

namespace Polecenie
{
    class Konto
    {
        private int numerKonta;
        private decimal saldo;

        public Konto(int numerKonta, decimal saldoPoczatkowe)
        {
            this.numerKonta = numerKonta;
            this.saldo = saldoPoczatkowe;
        }

        public void Wpłata(decimal kwota)
        {
            if (kwota < 0) throw new Exception("Kwota nie moze byc ujemna");
            saldo += kwota;
            Console.WriteLine("Nastapila wplata na konto {0:d} kwoty {1:c}. Saldo po operacji {2:c}.", numerKonta, kwota, saldo);
        }

        public bool Wypłata(decimal kwota)
        {
            if (kwota < 0) throw new Exception("Kwota nie moze byc ujemna");
            if (saldo >= kwota)
            {
                saldo -= kwota;
                Console.WriteLine("Nastapila wyplata z konta {0:d} kwoty {1:c}. Saldo po operacji {2:c}.", numerKonta, kwota, saldo);
                return true;
            }
            else return false;
        }

        public void WyświetlSaldo()
        {
            Console.WriteLine("Saldo konta {0:d}: {1:c}.", numerKonta, saldo);
        }

        public static bool Przelew(Konto kontoPłatnika, Konto kontoOdbiorcy, decimal kwota)
        {
            if (kontoPłatnika.numerKonta == kontoOdbiorcy.numerKonta) throw new Exception("Nie jest możliwe wykonanie przelewu na to samo konto");

            Console.WriteLine("Przygotowanie do przelewu z konta {0:d} na konto {1:d} kwoty {2:c}.", kontoPłatnika.numerKonta, kontoOdbiorcy.numerKonta, kwota);

            //niebezpieczne ze względu na dostęp z wielu wątków!!
            if (kontoPłatnika.Wypłata(kwota))
            {
                kontoOdbiorcy.Wpłata(kwota);
                Console.WriteLine("Wykonany zostal przelew z konta {0:d} na konto {1:d} kwoty {2:c}.", kontoPłatnika.numerKonta, kontoOdbiorcy.numerKonta, kwota);
                return true;
            }
            return false;
        }
    }

    /*
    interface IPolecenie
    {
        bool CzyMożnaWykonać();
        void Wykonaj();
    }
    */

    class PoleceniePrzelewu : ICommand
    {
        private Konto kontoPłatnika;
        private Konto kontoOdbiorcy;
        private decimal kwota;
        private bool wykonane = false;

        public PoleceniePrzelewu(Konto kontoPłatnika, Konto kontoOdbiorcy, decimal kwota, bool wykonane = false)
        {
            if (kontoPłatnika == null || kontoOdbiorcy == null || !(kwota > 0))
                throw new ArgumentException("Niepoprawne dane polecenia przelewu");
            this.kontoPłatnika = kontoPłatnika;
            this.kontoOdbiorcy = kontoOdbiorcy;
            this.kwota = kwota;
            this.wykonane = wykonane;
        }

        public event EventHandler CanExecuteChanged
        {
            add
            {
                CommandManager.RequerySuggested += value;
            }
            remove
            {
                CommandManager.RequerySuggested -= value;
            }
        }

        public bool CanExecute(object parameter)
        {
            return !wykonane && kwota > 0;
        }

        public void Execute(object parameter)
        {
            Konto.Przelew(kontoPłatnika, kontoOdbiorcy, kwota);
            wykonane = true;
        }
    }
}
